def linearRelation(df, pol1, pol2):
    """Plots the slope `m` by fitting a linear model between
    pol1 and plo2, i.e., plo1

    This plot given two pollutants will graph the linear 
    relationship. The df given will be converted to the slope
    between the pollutants. 
        
    Parameters
    ----------
    df: data frame
        minimally containing date and two pollutants
    pol1: type string
        First pollutant that when plotted would appear
        on the x-axis of a relationship, ex: 'nox'
    pol2: type string
        Second pollutant that when plotted would appear
        on the y-axis of a relationship, ex: 'pm10'     
    """
    import datetime as dt
    import matplotlib.pyplot as plt
    import matplotlib as mpl
    import numpy as np
    import pandas as pd
    from numpy import array
    import seaborn as sns

    df.index = pd.to_datetime(df.date)
    unique_years = np.unique(df.index.year)

    """
    Finds the unique years in the given data
    and converts it to a string type within a list 
    to be called on later to determine manipulation 
    of data
    """
    i = 0
    year = []
    while i < len(unique_years):
        year.append(str(unique_years[i]))
        i = i + 1
    # print(year)
    num_unique_years = len(year)
    df = df.drop("date", axis=1)

    """
    Will create a df(df_1) varying in lentgh based on the
    number of years that the data contains. The df 
    consists of the slope between the two given pollutants
    """
    i = 0
    values = []
    while i < num_unique_years:
        df_new = df[year[i]].resample("1D").mean()
        df_new = df_new.fillna(method="ffill")
        df_new["month"] = df_new.index.month
        # df_new.index.dayofweek
        pol_1 = df_new[pol1].mean()
        # this might not be entirely correct.
        # TODO: we want to take the average of the slope.
        # [Not take the average and find the slope]
        # Additionally, we would like to have the option of
        # changing the granularity.
        pol_2 = df_new[pol2].mean()
        values.append(pol_2 / pol_1)
        i = i + 1
    df_1 = pd.DataFrame(values)
    df_1.columns = ["div"]
    
    """ Creates a df for the first polutant that 
        contains the average value for every month
        of every year given
    """
    var1 = []
    var2 = []
    i = 0
    x = 0
    j = 0
    while i < num_unique_years:

        df_new = df[year[j]].resample("1D").mean()
        df_new = df_new.fillna(method="ffill")
        df_new["month"] = df_new.index.month
        df_new["day"] = df_new.index.dayofweek
        df_new["hour"] = df_new.index.hour
        i = i + 1
        j = j + 1
        x = 0
        while x < 12:
            a = df_new[df_new.month == x]
            mean_var1 = a[pol1].mean()
            var1.append(mean_var1)
            x = x + 1

    """ Creates a df for the second polutant that 
        contains the average value for every month
        of every year given
    """
    i = 0
    x = 0
    j = 0
    while i < num_unique_years:

        df_new = df[year[j]].resample("1D").mean()
        df_new = df_new.fillna(method="ffill")
        df_new["month"] = df_new.index.month
        df_new["day"] = df_new.index.dayofweek
        df_new["hour"] = df_new.index.hour
        i = i + 1
        j = j + 1
        x = 0
        while x < 12:
            a = df_new[df_new.month == x]
            mean_var2 = a[pol2].mean()
            var2.append(mean_var2)
            x = x + 1

    """ Finds the slope of the new dataframe
    """
    res = [i / j for i, j in zip(var2, var1)]
    df_2 = pd.DataFrame(res)
    df_2.columns = [pol2 + "/" + pol1]

    """ creates a list that contains the year values 
        of each data point to use for graphing a 
        scatterplot
    """
    scatterY = []
    t = 0
    while t < num_unique_years:
        r = 0
        while r < 12:
            scatterY.append(t + (r / 12))
            r = r + 1
        t = t + 1

    """ Plots a line plot of the average slope between the
        two pollutants given for every year. That plot is then superimposed 
        with a scatter plot of the average value of every month of every year
    """

    fig = plt.figure()
    ax = fig.add_subplot(111, label="1")
    ax2 = fig.add_subplot(111, label="2", frame_on=False)

    ax.plot(df_1, color="red", label = f"average slope between {pol1} & {pol2}")
    ax.set_xlabel("Year", color="red")
    ax.set_ylabel(f"Pollutant ratio ({pol2}/{pol1})", color="red")
    ax.tick_params(axis="x", colors="red")
    ax.tick_params(axis="y", colors="red")
    ax.set_xticklabels(unique_years)
  
    ax2.scatter(scatterY, res, color="blue", label = "average value of (" + pol2 + 
                "/" + pol1 + ") for every month of year")
    ax2.xaxis.tick_top()
    ax2.yaxis.tick_right()
    ax2.xaxis.set_label_position("top")
    ax2.yaxis.set_label_position("right")
    ax2.tick_params(axis="x", colors="blue")
    ax2.tick_params(axis="y", colors="blue")
    
    fig.legend(loc="upper right", bbox_to_anchor=(1.1,1.1))
    
    plt.show()


# =============================================================================
# df = pd.read_csv("mydata.csv")
# linearRelation(df,'nox','so2')
# =============================================================================



def interpolPlot(
    df,
    shape_df,
    long,
    lat,
    pollutant,
    Interpolator,
    resolution=100,
    partitions=15,
    cmap="inferno",
    Tcolor="k",
    markersize=3,
    plot_train_points=False,
    extrapolate=True,
):
    """Interpolates data at unknown locations--based on the passed
    `Interpolator`--and plots them on a geographical plot.

    Parameters
    ----------

    df: data frame 
        minimally containing latitude and longitude
        and pollutant value

    shape_df: geo pandas data frame 
       minimally containing DISTRICT    and geometry

    long: str
        name of column in df having longitudes

    lat: str
        name of column in df having latitudes

    pollutant: str
        name of column in df having pollutant values.

    Interpolator: sklean estimator
        Interpolator to be used to interpolate values.
        eg. Lasso from sklearn.linear_models

    resolution: int
        Resolution at which to interpolate. Bigger number 
        means more granularity.

    partitions: int
        Levels in the contour created.

    cmap: str or matplotlib.colors.ListedColormap
        cmap to be used while creating contours

    Tcolor: str
        Color to be used while plotting training points.

    markersize: float
        Training Point markersize. 
        (Also varies with the amount of pollution found)            

    plot_train_points: bool
        If True, plots, the training points used to interpolate.

    extrapolate: bool
        If False, limits interpolation within bounds of the
        training points used.
    """

    import geopandas
    import numpy as np
    import pandas as pd
    import matplotlib.pyplot as plt
    from geopandas import GeoDataFrame
    from matplotlib.colors import ListedColormap
    from shapely.geometry import Polygon, MultiPolygon, Point

    df = df.sample(frac=1)
    trainX = df[[long, lat]].values
    trainy = df[pollutant].values

    def bounds_minmax(bound1, bound2=None):
        if bound2 is None:
            return bound1
        temp = []
        temp.append(min(bound1[0], bound2[0]))
        temp.append(min(bound1[1], bound2[1]))
        temp.append(max(bound1[2], bound2[2]))
        temp.append(max(bound1[3], bound2[3]))
        return temp

    def collec_to_gdf(collec_poly):
        """Transform a `matplotlib.contour.QuadContourSet` to a GeoDataFrame"""
        polygons, colors = [], []
        for i, polygon in enumerate(collec_poly.collections):
            mpoly = []
            for path in polygon.get_paths():
                try:
                    path.should_simplify = False
                    poly = path.to_polygons()
                    # Each polygon should contain an exterior ring + maybe hole(s):
                    exterior, holes = [], []
                    if len(poly) > 0 and len(poly[0]) > 3:
                        # The first of the list is the exterior ring :
                        exterior = poly[0]
                        # Other(s) are hole(s):
                        if len(poly) > 1:
                            holes = [h for h in poly[1:] if len(h) > 3]
                    mpoly.append(Polygon(exterior, holes))
                except:
                    print("Warning: Geometry error when making polygon #{}".format(i))
            if len(mpoly) > 1:
                mpoly = MultiPolygon(mpoly)
                polygons.append(mpoly)
                colors.append(polygon.get_facecolor().tolist()[0])
            elif len(mpoly) == 1:
                polygons.append(mpoly[0])
                colors.append(polygon.get_facecolor().tolist()[0])
        if type(collec_poly.cmap) != ListedColormap:
            raise ValueError(
                """We only support ListedColormap right now.\n"""
                """simply convert your cmap to ListedColormap using url here."""
                """https://matplotlib.org/3.1.0/tutorials/colors/colormap-manipulation.html#sphx-glr-tutorials-colors-colormap-manipulation-py"""
            )

        ixs = [collec_poly.cmap.colors.index(c[:3]) for c in colors]
        return GeoDataFrame(
            geometry=polygons,
            data={"RGBA": colors, "cmapIX": ixs},
            crs={"init": "epsg:4269"},
        )

    z = trainy
    if not extrapolate:
        x1max, x2max = np.max(trainX, axis=0)
        x1min, x2min = np.min(trainX, axis=0)
    else:
        bounds = None
        for index, row in shape_df.iterrows():
            poly = row["geometry"]
            bounds = bounds_minmax(poly.bounds, bounds)

        x1max, x2max = bounds[2:]
        x1min, x2min = bounds[:2]
    xi = np.linspace(x1min, x1max, resolution)
    yi = np.linspace(x2min, x2max, resolution)
    Xi, Yi = np.meshgrid(xi, yi)
    t = Interpolator
    t.fit(trainX, z)

    zi = t.predict(np.asarray([Xi.ravel(), Yi.ravel()]).T)
    zi = zi.reshape(Xi.shape)

    vmin = zi.min()
    vmax = zi.max()
    collec_poly = plt.contourf(Xi, Yi, zi, partitions, vmin=vmin, vmax=vmax, cmap=cmap)
    plt.close()

    gdf = collec_to_gdf(collec_poly)
    # new min max vales in term of index of colors
    vmax2 = gdf.cmapIX.max()
    vmin2 = gdf.cmapIX.min()

    # intersection with shape_df
    inter = geopandas.overlay(shape_df, gdf, how="intersection")
    ax = inter.plot(
        column="cmapIX", cmap=cmap, figsize=(12, 12), vmax=vmax2, vmin=vmin2,
    )
    ax = shape_df.plot(ax=ax, color="none", edgecolor="k", figsize=(12, 12))
    # getting geodataframe of the train points
    if plot_train_points:
        geometry = [Point(xy) for xy in zip(df[long], df[lat])]
        geodf = geopandas.GeoDataFrame(df, crs={"init": "epsg:4269"}, geometry=geometry)

        # finding the intersection and plotting
        from geopandas.tools import sjoin

        inter2 = sjoin(geodf, shape_df)
        inter2.plot(
            ax=ax,
            color=Tcolor,
            label="Train points",
            markersize=markersize * df[pollutant].values,
        )

    plt.axis("off")
    fig = ax.get_figure()
    cax = fig.add_axes([0.9, 0.3, 0.03, 0.4])
    sm = plt.cm.ScalarMappable(cmap=cmap, norm=plt.Normalize(vmin=vmin, vmax=vmax))
    sm._A = []
    ax.legend()

    bounds = np.linspace(vmin, vmax, partitions)
    fig.colorbar(sm, cax=cax, ticks=bounds, 
        boundaries=bounds, format="%.2f")
    return ax



def selectByDate(df, year):
    """ 
    Utility function to cut given dataframe by the year 
    and find the average value of each day 
    
    Parameters
    ----------
    df: data frame
        a data frame containing a date field
    year: type string
        a year to select to cut data
    """
    import pandas as pd
    import numpy as np

    df.index = pd.to_datetime(df.date)
    df = df.drop("date", axis=1)
    df_n = df[year].resample("1D").mean()
    df_n = df_n.fillna(method="ffill")
    df_n["month"] = df_n.index.month
    df_n.index.dayofweek
    print(df_n)


# =============================================================================
# df = pd.read_csv("mydata.csv")
# selectByDate(df,'2003')
# =============================================================================



def summaryPlot(df):
    """ Plots import summary of data frame given. Plots line plots
        and histograms for each polutant as well as statiscs such as 
        mean,max,min,median, and 95th percentile
        
        Parameters
        ----------
        df: data frame
            data frame to be summarised. Must contain a date field
            and at least one other parameter 
    """
    import datetime as dt
    import matplotlib.pyplot as plt
    import matplotlib as mpl
    import numpy as np
    import pandas as pd
    from numpy import array
    import matplotlib.patches as mpatches
    import seaborn as sns
    from matplotlib.pyplot import figure

    pm10_s = 0
    pm10_m = 0
    pm10_h = 0
    pm25_s = 0
    pm25_m = 0
    pm25_h = 0
    so2_s = 0
    so2_m = 0
    so2_h = 0
    co_s = 0
    co_m = 0
    co_h = 0
    o3_s = 0
    o3_m = 0
    o3_h = 0
    no2_s = 0
    no2_m = 0
    no2_h = 0

    class color:
        # Allows for bolded and underlined text
        BOLD = "\033[1m"
        UNDERLINE = "\033[4m"
        END = "\033[0m"

    # Reads df and fills empty values
    df.index = pd.to_datetime(df.date)
    df = df.drop("date", axis=1)
    df_all = df.resample("1D")
    df_all = df
    df_all = df_all.fillna(method="ffill")

    dataPoints = ["pm25", "co", "so2", "pm10", "o3", "no2", "nox", "wd", "ws"]

    for column in df_all.columns:
        if str(column) == "pm10":
            df_pm10 = df_all["pm10"]
            for (_, data) in df_pm10.iteritems():
                if data < 100:
                    pm10_s += 1
                elif data > 100 and data < 250:
                    pm10_m += 1
                elif data > 250:
                    pm10_h += 1

        if str(column) == "pm25":
            df_pm25 = df_all["pm25"]
            for (_, data) in df_pm25.iteritems():
                if data < 60:
                    pm25_s += 1
                elif data > 60 and data < 90:
                    pm25_m += 1
                elif data > 100:
                    pm25_h += 1

        if str(column) == "co":
            df_co = df_all["co"]
            for (_, data) in df_co.iteritems():
                if data < 2:
                    co_s += 1
                elif data > 2 and data < 10:
                    co_m += 1
                elif data > 10:
                    co_h += 1

        if str(column) == "so2":
            df_so2 = df_all["so2"]
            for (_, data) in df_so2.iteritems():
                if data < 80:
                    so2_s += 1
                elif data > 80 and data < 380:
                    so2_m += 1
                elif data > 380:
                    so2_h += 1

        if str(column) == "o3":
            df_o3 = df_all["o3"]
            for (_, data) in df_o3.iteritems():
                if data < 100:
                    o3_s += 1
                elif data > 100 and data < 168:
                    o3_m += 1
                elif data > 168:
                    o3_h += 1

        if str(column) == "no2":
            df_no2 = df_all["no2"]
            for (_, data) in df_no2.iteritems():
                if data < 80:
                    no2_s += 1
                elif data > 80 and data < 180:
                    no2_m += 1
                elif data > 180:
                    no2_h += 1

    i = 0
    sub = 1
    while i < 9:
        # Plots line and histogram plots for ecery polutant
        # in the correct location based on subplot

        plt.figure(1, figsize=(25, 25))
        plt.subplot(9, 3, sub)
        sub = sub + 1
        a = df_all[dataPoints[i]].plot.line(color="gold")
        a.axes.get_xaxis().set_visible(False)
        a.yaxis.set_label_position("left")
        plt.ylabel(dataPoints[i], fontsize=30, bbox=dict(facecolor="whitesmoke"))

        plt.subplot(9, 3, sub)

        sub = sub + 1
        plt.hist(df_all[dataPoints[i]], bins=50, color="green")

        ax1 = plt.subplot(9, 3, sub)

        # fig1,ax1 = plt.subplots()

        sub = sub + 1

        labels = ["Safe", "Moderate", "High"]
        if dataPoints[i] == "pm10":
            sizes = [pm10_s, pm10_m, pm10_h]
        elif dataPoints[i] == "pm25":
            sizes = [pm25_s, pm25_m, pm25_h]
        elif dataPoints[i] == "so2":
            sizes = [so2_s, so2_m, so2_h]
        elif dataPoints[i] == "co":
            sizes = [co_s, co_m, co_h]
        elif dataPoints[i] == "o3":
            sizes = [o3_s, o3_m, o3_h]
        elif dataPoints[i] == "no2":
            sizes = [no2_s, no2_m, no2_h]

        explode = [0, 0, 1]
        ax1.pie(
            sizes,
            explode=explode,
            labels=labels,
            autopct="%1.1f%%",
            shadow=False,
            startangle=90,
        )
        ax1.axis("equal")

        sizes = [0, 0, 0]

        i = i + 1
    i = 0
    while i < 9:
        # Calculates statistics
        nDf = df[dataPoints[i]]
        missing = nDf.isna().sum() + sum(n < 0 for n in nDf)
        minVal = nDf.min()
        maxVal = nDf.max()
        meanVal = nDf.mean()
        medianVal = nDf.median()
        percentile = nDf.quantile(0.95)
        print("---------------")
        print(color.BOLD + color.UNDERLINE + dataPoints[i] + color.END)
        print("min = " + str(0))
        print("max = " + str(maxVal))
        print("missing = " + str(missing))
        print("mean = " + str(meanVal))
        print("median = " + str(medianVal))
        print("95th percentile = " + str(percentile))
        i = i + 1


# =============================================================================
# df = pd.read_csv('mydata.csv')
# summaryPlot(df)
# =============================================================================



def timeInteractiveScatterPlot(
    df,
    date_time_col_name,
    location_col_name,
    value_col_name,
    latitude_col_name,
    longitude_col_name,
    start_date,
    end_date,
    pollutant_ID,
    width=800,
    height=660,
    location=[22, 82],
    zoom_start=4.5,
    control_scale=True,
    tiles="Stamen Terrain",
    min_lat=6,
    max_lat=105,
    min_lon=68,
    max_lon=97,
    period="PT1M",
    date_options="YYYY-MM-DD HH:mm",
):

    """ Plots a time interactive scatter plot 
      of a given pollutant for given duration 
      of time.

  Parameters
  ----------
  df: data frame 
      minimally containing time stamps, location, 
      pollutant value, latitude and longitude

  date_time_col_name : type string
                       name of column in df having time stamps

  location_col_name : type string
                      name of column in df having station name

  value_col_name: type string
                  name of column in df having value of pollutant

  latitude_col_name : type string
                      name of column in df having latitute

  longitude_col_name : type string
                       name of column in df having longitude
                  
  start_date: type string
             time stamp corresponding to startdate,
             ex : '2019-08-01 00:00:00+05:30'

  end_date : type string
             time stamp corresponding to end date,
             ex : '2019-08-05 00:00:00+05:30'

  pollutant_ID: type integer
              A pollutant ID correspoinding to 
              a pollutant, ex: 1 for PM2.5
  """

    import numpy as np
    import matplotlib.pyplot as plt
    import pandas as pd
    import seaborn as sns
    import folium
    from folium.plugins import TimestampedGeoJson
    import branca.colormap as cm
    import json

    pollutants = {
        1: {
            "notation": "PM2.5",
            "name": "Particulate matter < 10 µm",
            "bin_edges": np.array([10, 20, 30, 40, 50, 70, 100, 150, 200]),
        },
        2: {
            "notation": "SO2",
            "name": "Sulphur dioxide",
            "bin_edges": np.array([15, 30, 45, 60, 80, 100, 125, 165, 250]),
        },
        3: {
            "notation": "PM10",
            "name": "Particulate matter < 10 µm",
            "bin_edges": np.array([10, 20, 30, 40, 50, 70, 100, 150, 200]),
        },
        4: {
            "notation": "O3",
            "name": "Ozone",
            "bin_edges": np.array([30, 50, 70, 90, 110, 145, 180, 240, 360]),
        },
        5: {
            "notation": "NO2",
            "name": "Nitrogen dioxide",
            "bin_edges": np.array([25, 45, 60, 80, 110, 150, 200, 270, 400]),
        },
        6: {
            "notation": "CO",
            "name": "Carbon monoxide",
            "bin_edges": np.array([1.4, 2.1, 2.8, 3.6, 4.5, 5.2, 6.6, 8.4, 13.7]),
        },
    }

    def color_coding(poll, bin_edges):
        """ Maps polluatnt value to the bins
    """
        idx = np.digitize(poll, bin_edges, right=True)
        return color_scale[idx]

    def prepare_data(df):
        """ Returns a df for selected time range
        indexed by location and time stamp.
        Adds a color column according to the pollutant value.
    """
        print("Preparing the data frame...")
        df[date_time_col_name] = pd.to_datetime(df[date_time_col_name])
        df = df[
            (df[date_time_col_name] >= start_date)
            & (df[date_time_col_name] <= end_date)
        ]

        df = df.set_index(keys=[location_col_name, date_time_col_name])
        df = df.sort_index()

        df[value_col_name] = df.groupby(level=0)[value_col_name].bfill().fillna(0)
        df["color"] = df[value_col_name].apply(
            color_coding, bin_edges=pollutants[pollutant_ID]["bin_edges"]
        )
        print(df.head(5))
        return df

    def create_geojson_features(df):
        """ Creates a Geo JSON
    """
        print("Creating Geo JSON...")
        features = []
        for _, row in df.iterrows():
            feature = {
                "type": "Feature",
                "geometry": {
                    "type": "Point",
                    "coordinates": [row[longitude_col_name], row[latitude_col_name]],
                },
                "properties": {
                    "popup": row[location_col_name].__str__()
                    + "\n"
                    + pollutants[pollutant_ID]["notation"]
                    + " "
                    + row[value_col_name].__str__()
                    + " µg/m³",
                    "time": row[date_time_col_name].__str__(),
                    "style": {"color": row["color"]},
                    "icon": "circle",
                    "iconstyle": {
                        "fillColor": row["color"],
                        "fillOpacity": 0.8,
                        "stroke": "true",
                        "radius": 7,
                    },
                },
            }
            features.append(feature)
        return features

    def make_map(features):
        """ Returns a HTML file with time interactive scatter plot
    """
        print("Plotting the map...")
        pollution_map = folium.Map(
            width=width,
            height=height,
            location=location,
            zoom_start=zoom_start,
            control_scale=True,
            tiles=tiles,
            min_lat=min_lat,
            max_lat=max_lat,
            min_lon=min_lon,
            max_lon=max_lon,
        )

        colormap.add_to(pollution_map)

        TimestampedGeoJson(
            {"type": "FeatureCollection", "features": features},
            period=period,
            add_last_point=True,
            auto_play=False,
            loop=False,
            max_speed=1,
            loop_button=True,
            date_options=date_options,
            time_slider_drag_update=True,
        ).add_to(pollution_map)
        print("Saved the map.")
        return pollution_map

    # =============================================================================
    #     Creates a colormap for legend.
    # =============================================================================
    color_scale = np.array(
        [
            "#053061",
            "#2166ac",
            "#4393c3",
            "#92c5de",
            "#d1e5f0",
            "#fddbc7",
            "#f4a582",
            "#d6604d",
            "#b2182b",
            "#67001f",
        ]
    )
    colormap = cm.LinearColormap(colors=color_scale, vmin=0, vmax=200)
    colormap = colormap.to_step(index=pollutants[pollutant_ID]["bin_edges"])
    colormap.caption = pollutants[pollutant_ID]["notation"] + " gradient scale"

    # =============================================================================
    #     Prepares the data frame.
    #     Creates a Geo JSON to be added to map.
    # =============================================================================
    clean_df = prepare_data(df)
    clean_df = clean_df.reset_index()

    features = create_geojson_features(clean_df)
    map_ = make_map(features)

    return map_



class Legend:
    """Utility class for getting AQ standards.
  """

    import numpy as np
    import branca.colormap as cm

    country_pollutants = {
        "Canada": {
            "AQHI": {
                "notation": "Air Quality Health Index",
                "bin_edges": np.array([1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 20]),
                "color_scale": np.array(
                    [
                        "#00ccff",
                        "#0099cc",
                        "#006699",
                        "#ffff00",
                        "#ffcc00",
                        "#ff9933",
                        "#ff6666",
                        "#ff0000",
                        "#cc0000",
                        "#990000",
                        "#660000",
                    ]
                ),
            }
        },
        "Hong Kong": {
            "AQHI": {
                "notation": "Air Quality Health Index",
                "bin_edges": np.array([3, 6, 7, 10, 20]),
                "color_scale": np.array(
                    ["#008000", "#FFA500", "#FF0000", "#A52A2A", "#000000"]
                ),
            }
        },
        "Mainland China": {
            "AQI": {
                "notation": "Air Quality Index",
                "bin_edges": np.array([50, 100, 150, 200, 300, 450]),
                "color_scale": np.array(
                    ["#00FF00", "#FFFF00", "#FF9900", "#FF0000", "#540099", "#800000"]
                ),
            }
        },
        "South Korea": {
            "CAI": {
                "notation": "Comprehensive Air-quality Index",
                "bin_edges": np.array([50, 100, 150, 200, 350, 500]),
                "color_scale": np.array(
                    ["#0000FF", "#005500", "#FFFF00", "#FF9900", "#FF0000", "#990000"]
                ),
            }
        },
        "United Kingdom": {
            "APB": {
                "notation": "Air Pollution Banding",
                "bin_edges": np.array([1, 2, 3, 4, 5, 6, 7, 8, 9, 10]),
                "color_scale": np.array(
                    [
                        "#CCFFCC",
                        "#66FF66",
                        "#00FF00",
                        "#99FF00",
                        "#FFFF00",
                        "#FFCC00",
                        "#FF6600",
                        "#FF3300",
                        "#FF0000",
                        "#FF0066",
                    ]
                ),
            }
        },
        "India": {
            "AQI": {
                "notation": "Air Quality Index",
                "bin_edges": np.array([50, 100, 200, 300, 400, 500]),
                "color_scale": np.array(
                    ["#00CC00", "#66CC00", "#FFFF00", "#FF9900", "#FF0000", "#A52A2A"]
                ),
            },
            "so2": {
                "notation": "Sulphur dioxide",
                "bin_edges": np.array([40, 80, 380, 800, 1600]),
                "color_scale": np.array(
                    ["#00CC00", "#66CC00", "#FFFF00", "#FF9900", "#FF0000", "#A52A2A"]
                ),
            },
            "pm10": {
                "notation": "Particulate matter < 10 µm",
                "bin_edges": np.array([50, 100, 250, 350, 430]),
                "color_scale": np.array(
                    ["#00CC00", "#66CC00", "#FFFF00", "#FF9900", "#FF0000", "#A52A2A"]
                ),
            },
            "pm25": {
                "notation": "Particulate matter < 2.5 µm",
                "bin_edges": np.array([30, 60, 90, 120, 250]),
                "color_scale": np.array(
                    ["#00CC00", "#66CC00", "#FFFF00", "#FF9900", "#FF0000", "#A52A2A"]
                ),
            },
            "o3": {
                "notation": "Ozone",
                "bin_edges": np.array([50, 100, 168, 208, 748]),
                "color_scale": np.array(
                    ["#00CC00", "#66CC00", "#FFFF00", "#FF9900", "#FF0000", "#A52A2A"]
                ),
            },
            "no2": {
                "notation": "Nitrogen Dioxide",
                "bin_edges": np.array([40, 80, 180, 280, 400]),
                "color_scale": np.array(
                    ["#00CC00", "#66CC00", "#FFFF00", "#FF9900", "#FF0000", "#A52A2A"]
                ),
            },
            "co": {
                "notation": "Carbon Monoxide",
                "bin_edges": np.array([1, 2, 10, 17, 34]),
                "color_scale": np.array(
                    ["#00CC00", "#66CC00", "#FFFF00", "#FF9900", "#FF0000", "#A52A2A"]
                ),
            },
            "nh3": {
                "notation": "Ammonia",
                "bin_edges": np.array([200, 400, 800, 1200, 1800]),
                "color_scale": np.array(
                    ["#00CC00", "#66CC00", "#FFFF00", "#FF9900", "#FF0000", "#A52A2A"]
                ),
            },
            "pb": {
                "notation": "Lead",
                "bin_edges": np.array([0.5, 1, 2, 3, 3.5]),
                "color_scale": np.array(
                    ["#00CC00", "#66CC00", "#FFFF00", "#FF9900", "#FF0000", "#A52A2A"]
                ),
            },
        },
        "Europe": {
            "CAQI": {
                "notation": "Common Air Quality Index (CAQI)",
                "bin_edges": np.array([25, 50, 75, 100, 200]),
                "color_scale": np.array(
                    ["#79BC6A", "#BBCF4C", "#EEC20B", "#F29305", "#E8416F"]
                ),
            }
        },
        "USA": {
            "AQI": {
                "notation": "Air Quality Index",
                "bin_edges": np.array([50, 100, 150, 200, 300, 500]),
                "color_scale": np.array(
                    ["#00E400", "#FFFF00", "#FF7E00", "#FF0000", "#99004C", "#7E0023"]
                ),
            }
        },
    }

    def pollutant_legend(country, pollutant):
        colormap = Legend.cm.LinearColormap(
            colors=Legend.country_pollutants[country][pollutant]["color_scale"],
            vmin=0,
            vmax=Legend.country_pollutants[country][pollutant]["bin_edges"][-1],
            index=Legend.country_pollutants[country][pollutant]["bin_edges"],
        )
        return colormap

    def pollutant_color_coding(poll, country, pollutant):
        bin_edges = Legend.country_pollutants[country][pollutant]["bin_edges"]
        idx = np.digitize(poll, bin_edges, right=True)
        return Legend.country_pollutants[country][pollutant]["color_scale"][idx]



# # -*- coding: utf-8 -*-
# from pkg_resources import get_distribution, DistributionNotFound

# try:
#     # Change here if project is renamed and does not equal the package name
#     dist_name = "vayu"
#     __version__ = get_distribution(dist_name).version
# except DistributionNotFound:
#     __version__ = "unknown"
# finally:
#     del get_distribution, DistributionNotFound

from .timeProp import timeProp
from .timePlot import timePlot
from .windRosePy import windRose
from .trendLevel import trendLevel
from .summaryPlot import summaryPlot
from .scatterPlot import scatterPlot
from .smoothTrend import smoothTrend
from .selectByDate import selectByDate
from .calendarPlot import calendarPlot
from .interpolPlot import interpolPlot
from .timeVariation import timeVariation
from .linearRelation import linearRelation
from .interactiveChoroplethPlot import interactiveChoroplethPlot
from .timeInteractiveScatterPlot import timeInteractiveScatterPlot



def timeProp(df, year, pollutant, 
    avg_days, date_time_col_name, 
    sorted_bars=True):
    """
    Plot a stacked bar graph of all data in the df
    based on frequency of wind direction in compass
    directions. Takes the average of every 3 days
    in each bar. The hight of the bar is value of 
    the pollutant that 3 day period. The bars are 
    binned proportionaly based on the overall value of the 
    pollutant 
    
    Parameters
    ----------
    df: data frame
        data frame which has the fields of date and the pollutant
        to be graphed
    year: type string
        The year of which the data will be cut
    pollutant: type string
        The pollutant of which to plot
    avg_days: type integer
      number of days to take the average of (controlling 
      granularity of the X-axis)
    date_time_col_name: type string
      df column name containing date-time stamps  
    """
    import datetime as dt
    import matplotlib.pyplot as plt
    import matplotlib as mpl
    import numpy as np
    import pandas as pd
    from numpy import array
    import matplotlib.patches as mpatches

    # =============================================================================
    #     Cuts data into the year specified and averages the
    #     values of each day
    # =============================================================================
    df.index = pd.to_datetime(df[date_time_col_name])
    df = df.drop(date_time_col_name, axis=1)
    df_year = df[year]
    df_year = df_year.fillna(method="ffill")
    df_year["month"] = df_year.index.month

    # New df containing only the values of the pollutant specified
    polArray = df[year].resample("1D").mean()
    polArray = df_year[pollutant]

    nA = []
    neA = []
    eA = []
    seA = []
    sA = []
    swA = []
    wA = []
    nwA = []
    polMeanS = 0
    polMeanE = avg_days
    dfStart = 0
    dfEnd = 24 * avg_days  # 24*3 for 3 day average

    x = 0

    while x < int(
        365 / avg_days
    ):  # example: 365 days / 3 = 121 floored. Represents number of bars total
        n = 0
        ne = 0
        e = 0
        se = 0
        s = 0
        sw = 0
        w = 0
        nw = 0
        a = df_year[dfStart:dfEnd]
        b = a["wd"]

        i = 0
        while (
            i < 24 * avg_days
        ):  # Bins the wd data into categories for stacked bar graph
            if b[i] > 348.75 or b[i] < 33.75:
                n = n + 1
            elif b[i] > 33.75 and b[i] < 78.75:
                ne = ne + 1
            elif b[i] > 78.75 and b[i] < 123.75:
                e = e + 1
            elif b[i] > 123.75 and b[i] < 168.75:
                se = se + 1
            elif b[i] > 168.75 and b[i] < 213.75:
                s = s + 1
            elif b[i] > 213.75 and b[i] < 258.75:
                sw = sw + 1
            elif b[i] > 258.75 and b[i] < 303.75:
                w = w + 1
            elif b[i] > 303.75 and b[i] < 348.75:
                nw = nw + 1

            i = i + 1
        # calculates the 3 day proportion mean of each polutant and stores
        # it in a new list
        n = (n / 24 * avg_days) * (polArray[polMeanS:polMeanE].mean())
        ne = (ne / 24 * avg_days) * (polArray[polMeanS:polMeanE].mean())
        e = (e / 24 * avg_days) * (polArray[polMeanS:polMeanE].mean())
        se = (se / 24 * avg_days) * (polArray[polMeanS:polMeanE].mean())
        s = (s / 24 * avg_days) * (polArray[polMeanS:polMeanE].mean())
        sw = (sw / 24 * avg_days) * (polArray[polMeanS:polMeanE].mean())
        w = (w / 24 * avg_days) * (polArray[polMeanS:polMeanE].mean())
        nw = (nw / 24 * avg_days) * (polArray[polMeanS:polMeanE].mean())

        nA.append(n)
        neA.append(ne)
        eA.append(e)
        seA.append(se)
        sA.append(s)
        swA.append(sw)
        wA.append(s)
        nwA.append(nw)
        x = x + 1
        # Adds to start and end values to get through end of df
        polMeanS = polMeanS + avg_days
        polMeanE = polMeanE + avg_days
        dfStart = dfStart + 24 * avg_days
        dfEnd = dfEnd + 24 * avg_days

    #########################################

    # Plots the stacked bar graph with specific color represtations.
    # A legend is also plotted
    color_list = ["red", "blue", "green", "purple", "orange", "yellow", "brown", "pink"]

    X = np.arange(int(365 / avg_days))
    data = np.array([nA, neA, eA, seA, sA, swA, wA, nwA])

    if sorted_bars:
        for ix, x in enumerate(X):
            vals = data[:, ix]  # pollutant contributions (len == 8)
            assert len(vals) == len(color_list)
            # sorting based on the contributions
            col = sorted(zip(vals, color_list), key=lambda x: x[0], reverse=True)
            for i in range(data.shape[0]):
                plt.bar(
                    x, col[i][0], bottom=np.sum([c[0] for c in col[:i]]), color=col[i][1],
                )
    else:
        X = np.arange(data.shape[1])
        for i in range(data.shape[0]):
            plt.bar(
                X,
                data[i],
                bottom=np.sum(data[:i], axis=0),
                color=color_list[i % len(color_list)],
            )

    # Legend stuff.
    red_patch = mpatches.Patch(color="red", label="North")
    blue_patch = mpatches.Patch(color="blue", label="North East")
    green_patch = mpatches.Patch(color="green", label="East")
    purple_patch = mpatches.Patch(color="purple", label="South East")
    orange_patch = mpatches.Patch(color="orange", label="South")
    yellow_patch = mpatches.Patch(color="yellow", label="South West")
    brown_patch = mpatches.Patch(color="brown", label="West")
    pink_patch = mpatches.Patch(color="pink", label="North West")
    plt.legend(
        bbox_to_anchor=(1.05, 1),
        loc=2,
        borderaxespad=0.0,
        handles=[
            red_patch,
            blue_patch,
            green_patch,
            purple_patch,
            orange_patch,
            yellow_patch,
            brown_patch,
            pink_patch,
        ],
    )

    # Setting fonts
    plt.xticks(
        np.arange(0, int(365 / avg_days), 30 / avg_days),
        (
            "Jan",
            "Feb",
            "Mar",
            "Apr",
            "May",
            "Jun",
            "Jul",
            "Aug",
            "Sep",
            "Oct",
            "Nov",
            "Dec",
        ),
    )
    plt.title("Wind Direction")
    plt.xlabel("Contribution weighted by Mean")
    plt.ylabel("Pollutant (µg/m³)")
    plt.rcParams["figure.figsize"] = 20, 6
    plt.show()


# =============================================================================
# mydata = pd.read_csv('mydata.csv')
# timeProp(mydata,'2003','so2')
# =============================================================================



def timeVariation(df, pollutant):
    """ 
    Plots four plots:
    - The average pollutant level per day by 
    each hour for each day of the week across all of the data
    - The average pollutant level by each hour, 
    across all data
    - The average pollutant level by each month of the
    year for across data
    - The average pollutant level per day of the week 
    across all data
    
    Parameters
    ----------
    df: pandas.DataFrame
      data frame of hourly data. 
      Must include a date field and at least one variable to plot
    pollutant: str
      Name of variable to plot

    """
    import datetime as dt
    import matplotlib.pyplot as plt
    import matplotlib as mpl
    import numpy as np
    import pandas as pd
    from numpy import array

    df["date"] = pd.to_datetime(df.date)

    df_days = df
    df_days["day"] = df_days["date"].dt.day_name()
    df_days = df_days.set_index(keys=["day"])
    df_days = df_days.groupby(["day"])

    dayWeek = [
        "Monday",
        "Tuesday",
        "Wednesday",
        "Thursday",
        "Friday",
        "Saturday",
        "Sunday",
    ]
    pollutant = ["pm10", "no2", "pm25", "so2"]

    for i in range(len(dayWeek)):
        plt.figure(1, figsize=(40, 5))
        plt.subplot(1, 7, i + 1)
        plt.grid()

        df_day = df_days.get_group(dayWeek[i])
        df_day["hour"] = df_day["date"].dt.hour

        df_day_m = df_day.groupby("hour").mean()
        df_day_m = df_day_m.reset_index()

        df_day_s = df_day.groupby("hour").std()
        df_day_s = df_day_s.reset_index()

        for k in range(len(pollutant)):
            plt.plot(df_day_m["hour"], df_day_m[pollutant[k]], label=pollutant[k])
            plt.fill_between(
                df_day_s["hour"],
                df_day_m[pollutant[k]] - 0.5 * df_day_s[pollutant[k]],
                df_day_m[pollutant[k]] + 0.5 * df_day_s[pollutant[k]],
                alpha=0.2,
            )
            plt.xlabel(dayWeek[i])
            plt.legend()

    plt.figure(2, figsize=(35, 5))
    plt.subplot(1, 3, 1)

    df_hour = df
    df_hour["hour"] = df_hour["date"].dt.hour

    df_hour_m = df.groupby("hour").mean()
    df_hour_m = df_hour_m.reset_index()

    df_hour_s = df.groupby("hour").std()
    df_hour_s = df_hour_s.reset_index()

    plt.grid()

    for i in range(len(pollutant)):
        plt.plot(df_hour_m["hour"], df_hour_m[pollutant[i]], label=pollutant[i])
        plt.fill_between(
            df_hour_s["hour"],
            df_hour_m[pollutant[i]] - 0.5 * df_hour_s[pollutant[i]],
            df_hour_m[pollutant[i]] + 0.5 * df_hour_s[pollutant[i]],
            alpha=0.2,
        )
        plt.xlabel("Hour")
        plt.legend()

    plt.subplot(1, 3, 2)
    df_month = df
    df_month["month"] = df_month["date"].dt.month

    df_month_m = df_month.groupby("month").mean()
    df_month_m = df_month_m.reset_index()

    df_month_s = df_month.groupby("month").std()
    df_month_s = df_month_s.reset_index()

    plt.grid()

    for i in range(len(pollutant)):
        plt.plot(df_month_m["month"], df_month_m[pollutant[i]], label=pollutant[i])
        plt.fill_between(
            df_month_s["month"],
            df_month_m[pollutant[i]] - 0.5 * df_month_s[pollutant[i]],
            df_month_m[pollutant[i]] + 0.5 * df_month_s[pollutant[i]],
            alpha=0.2,
        )
        plt.xlabel("Month")
        plt.legend()

    plt.subplot(1, 3, 3)
    df_weekday = df
    df_weekday["weekday"] = df_weekday["date"].dt.weekday

    df_weekday_m = df_weekday.groupby("weekday").mean()
    df_weekday_m = df_weekday_m.reset_index()

    df_weekday_s = df_weekday.groupby("weekday").std()
    df_weekday_s = df_weekday_s.reset_index()

    plt.grid()

    for i in range(len(pollutant)):
        plt.plot(
            df_weekday_m["weekday"], df_weekday_m[pollutant[i]], label=pollutant[i]
        )
        plt.fill_between(
            df_weekday_s["weekday"],
            df_weekday_m[pollutant[i]] - 0.5 * df_weekday_s[pollutant[i]],
            df_weekday_m[pollutant[i]] + 0.5 * df_weekday_s[pollutant[i]],
            alpha=0.2,
        )
        plt.xlabel("WeekDay")
        plt.legend()


# =============================================================================
# df = pd.read_csv("mydata.csv")
# timeVariation(df,['pm10'])
# =============================================================================



def googleMaps(df, lat, long, pollutant, dataLoc):
    """Plots a geographical plot.

    Plots a folium plot of longitude and latitude points 
    provided with information about each point when clicked 
    on.

    Parameters
    ----------
    df: pandas.DataFrame
        minimally containing date and values of pollutant, city,
        longitude, latitude, and AQI
    lat: str
        Name of column in df of where latitude points are
    long: str
        Name of column in df of where longitude points are
    pollutant: str
        Name of pollutant 
    dataLoc: str
        Name of df column where pollutanat values are stored

    """
    import folium
    import webbrowser
    import pandas as pd
    import matplotlib.pyplot as plt
    import numpy as np
    import pandas as pd

    latitude = 37.0902
    longitude = -95.7129
    Arithmetic_Mean_map = folium.Map(location=[latitude, longitude], zoom_start=4)

    # =============================================================================
    # df = pd.read_csv('interpolData.csv')
    # =============================================================================

    some_value = pollutant
    df = df.loc[df["Parameter Name"] == some_value]

    some_value = "2018-05-07"
    df = df.loc[df["Date Local"] == some_value]

    df = df.sample(frac=1)

    # df_train, df_test = train_test_split(df, test_size=0.2)
    df["Arithmetic Mean Q"] = pd.qcut(df[dataLoc], 4, labels=False)
    colordict = {0: "lightblue", 1: "lightgreen", 2: "orange", 3: "red"}

    for lat, lon, Arithmetic_Mean_Q, Arithmetic_Mean, city, AQI in zip(
        df[lat],
        df[long],
        df["Arithmetic Mean Q"],
        df[dataLoc],
        df["City Name"],
        df["AQI"],
    ):
        folium.CircleMarker(
            [lat, lon],
            radius=0.15 * AQI,
            popup=(
                "City: "
                + str(city).capitalize()
                + "<br>"
                #'Bike score: ' + str(bike) + '<br>'
                "Arithmetic_Mean level: "
                + str(Arithmetic_Mean)
                + "%"
            ),
            color="b",
            key_on=Arithmetic_Mean_Q,
            threshold_scale=[0, 1, 2, 3],
            fill_color=colordict[Arithmetic_Mean_Q],
            fill=True,
            fill_opacity=0.7,
        ).add_to(Arithmetic_Mean_map)
    Arithmetic_Mean_map.save("mymap.html")


# df = pd.read_csv('interpolData.csv')
# googleMaps(df,'Latitude','Longitude','Ozone','Arithmetic Mean')



# -*- coding: utf-8 -*-

"""
Created on Mon Aug 26 15:30:00 2019
@author: Tanmay Srivastava, Prerna Khanna
"""
#!/usr/bin/env python

def interactiveChoroplethPlot(
  gdf,
  df,
  pollutant,
  country,
  date_time_col_name,
  value_col_name,
  dist_col_name,
  start_date,
  end_date,
  opacity = 0.4
  ):
  
  """ Plots an Interactive Choropleth Plot
      of a given pollutant for given duration 
      of time.

  Parameters
  ----------

  gdf: geo pandas data frame 
       minimally containing DISTRICT	and geometry

  df: data frame 
      minimally containing time stamps and pollutant value

  pollutant: type string
             Name of pollutant ex: 'pm25'
  
  India: type string
         Name of country ex: 'India'            

  date_time_col_name: type string
                      name of column in df having time stamps

  value_col_name: type string
                  name of column in df having value of pollutant concentration
   
  dist_col_name: type string
                 name of column in df having district number            
  
  start_date: type string
              time stamp corresponding to startdate,
              ex : '2019-08-20-06:00'

  end_date: type string
            time stamp corresponding to end date,
            ex : '2019-08-20-23:00'
  
  opacity: type float
           A float value in range 0 to 1
           1 represents maximum opacity           
  """
  import plotly.graph_objects as go
  import plotly
  from folium import IFrame
  import matplotlib.pyplot as plt
  import geopandas as gpd
  import pandas as pd
  import numpy as np
  import seaborn as sns
  import folium
  import branca.colormap as cm
  import json
  from string import Template
  from folium.plugins import TimeSliderChoropleth
  from copy import deepcopy
  import os
  from .utils import Legend

  def color_coding(poll, bin_edges):
    """ Maps polluatnt value to the bins
    """
    idx = np.digitize(poll, bin_edges, right=True)
        
    return colors[idx] 
  
  def prepare_df(df):
    """ Returns a df for selected time range 
        and coverts timestamp to date time format.
        Adds a color column according to the pollutant value.
    """
    df = df[
            (df[date_time_col_name] >= start_date)
            & (df[date_time_col_name] <= end_date)
            ]
    time_ = df[date_time_col_name]
    df[date_time_col_name] = pd.to_datetime(time_).dt.strftime('%s')
    df = df.set_index(keys = [dist_col_name, date_time_col_name])
    df["color"] = df[value_col_name].apply(
        color_coding, bin_edges = cvals
    )
    df = df.reset_index()
    print(df.head())
    
    return df
    
  def create_popup_df(plot):
    """ Returns a df for selected time range.
        Adds a color column according to the pollutant value.
    """
    plot = plot[
            (plot[date_time_col_name] >= start_date)
            & (plot[date_time_col_name] <= end_date)
            ]
    plot = plot.set_index(keys = [dist_col_name, date_time_col_name])
    plot["color"] = plot[value_col_name].apply(
        color_coding, bin_edges = cvals
    )
    plot = plot.reset_index()
    
    return plot

  def prepare_style_dict(df, opacity):
    """
    Prepares a dictionary in the given format:

    styledict = {
            '0': {
            '2017-1-1': {'color': 'ffffff', 'opacity': 1}
            '2017-1-2': {'color': 'fffff0', 'opacity': 1}
            ...
            },
            ...,
            'n': {
            '2017-1-1': {'color': 'ffffff', 'opacity': 1}
            '2017-1-2': {'color': 'fffff0', 'opacity': 1}
            ...
            }
          }"""  
                
    features = {}
    for _, row in df.iterrows():  

      if row[dist_col_name] in features:    
        features[row[dist_col_name]].update({row[date_time_col_name]: {'color': row['color'],'opacity': opacity}
                                  })   
      else:
        feature = {
            row[dist_col_name] : {
                row[date_time_col_name]: {'color': row['color'],'opacity': opacity}
            }
        }    
        features.update(feature)  

    return features 

  def create_popup(index, district):
    """Creates an interactive timeseries popup for
       each district.    
    """
    district = str(district)
    i = index *97
    j = i + 97
    pop = plot[i:j]

    fig = go.Figure([go.Scatter(x=pop[date_time_col_name], y=pop[value_col_name])])
      
    fig.update_layout(
        title = Legend.country_pollutants[country][pollutant]["notation"],            
        xaxis_title="Time Stamp",
      yaxis_title="Pollutant Concentration (µg/m³)"
      )
    fig.update_xaxes(showticklabels=False)

    script = plotly.offline.plot(fig, include_plotlyjs=False, output_type='div')  

    html_start = """
    <html>
    <head>
      <script src="https://cdn.plot.ly/plotly-latest.min.js"></script>
    </head>
    <body>
    <h1>"%s"</h1>""" %district

    html_end = """
    </body>
    </html>"""

    html_final = html_start + script + html_end

    resolution, width, height = 75, 7, 5
    iframe = IFrame(html_final, width=(width*resolution)+75, height=(height*resolution)+50)
  
    return iframe 
  
  def make_map(gdf, geo, features):
    """ Returns a HTML file with time interactive choropleth plot
    """
    m = folium.Map([22, 82], tiles='Stamen Terrain', zoom_start=5)

    g = TimeSliderChoropleth(
        geo,
        styledict = features,
        ).add_to(m)

    colormap.add_to(m)
    style = {'fillColor': '#00000000', 'color': '#00000000'}

    #### adding tooltip #####
    '''folium.GeoJson(geo, tooltip = folium.features.GeoJsonTooltip(
        fields=['DISTRICT']),style_function=lambda x: style,
        ).add_to(m)'''

    for index, row in gdf.iterrows():  
      gs = folium.GeoJson(row['geometry'],style_function=lambda x: style)
      label = create_popup(index, row['DISTRICT'])

      folium.Popup(label, max_width=2560).add_to(gs)
      gs.add_to(m)
      
      if(index%20 == 0):
        print("Done for Districts ", index, " out of 640")
    
    m.save('TimeSliderChoropleth.html')
    print("Map created")
    return m

  def change_style(map_):
    """Changes time slider style.
       Makes the slider background transparent.
    """
    style_add = """
    <style>
      .slidecontainer {
        width: 90%;
      }
      
      .slider {
      -webkit-appearance: none;
      width: 100%;
      height: 15px;
      border-radius: 5px;  
      background: #383838;
      outline: none;
      opacity: 0.7;
      -webkit-transition: .2s;
      transition: opacity .2s;
    }

    .slider::-webkit-slider-thumb {
      -webkit-appearance: none;
      appearance: none;
      width: 25px;
      height: 25px;
      border-radius: 50%; 
      background: #02bd09;
      cursor: pointer;
    }

    .slider::-moz-range-thumb {
      width: 25px;
      height: 25px;
      border-radius: 50%;
      background: #02bd09;
      cursor: pointer;
    }
      </style>
    """
    f = open(os.path.join(os.getcwd(), 'TimeSliderChoropleth.html'),"r").read()
    insert_pos_1 =f.find("""<script src="https://d3js.org/d3.v4.min.js"></script>""")
    g = deepcopy(f)
    g = f[:insert_pos_1] + style_add + f[insert_pos_1:]

    style_2 = """.attr('class', 'slider')"""
    insert_pos_2 = g.find(""".style('align', 'center');""")
    j = deepcopy(g)
    j = g[:insert_pos_2] + style_2 + g[insert_pos_2:]

    '''style_3 = """position: absolute;"""
    insert_pos_3 = j.find("""  
        width: 100.0%;
        height: 100.0%;
        left: 0.0%;
        top: 0.0%;
        }
        """
    )
    l = deepcopy(j)
    l = j[:insert_pos_3] + style_3 + j[insert_pos_3:]  

    style_4 = """          
          #slider {
            position:relative;
            z-index:99999;
          }
          #slider-value 
          {
            position:relative;
            z-index:99999; 
          }         
        """
    insert_pos_4 = l.find("""
    </style>

    <style>
      .slidecontainer {
        width: 90%;
      }
    """
    ) 
    k = deepcopy(l)
    k = l[:insert_pos_4] + style_4 + l[insert_pos_4:] ''' 
         
    with open("TimeSliderChoropleth.html","w") as h:
      h.write(j)
    print("Map saved")  


  # =============================================================================
  #     Creates a colormap for legend.
  # =============================================================================

  cvals  = Legend.country_pollutants[country][pollutant]['bin_edges']
  colors = Legend.country_pollutants[country][pollutant]['color_scale']

  colormap = cm.LinearColormap(colors=colors, vmin=0, vmax=600, index = cvals)
  colormap.caption = Legend.country_pollutants[country][pollutant]["notation"] + " gradient scale"

  # =============================================================================
  #     Converts geo pandas df to json format
  # =============================================================================  
  geo = json.loads(gdf.to_json())
  plot = df

  # =============================================================================
  #     Prepares the Dataframe
  # ============================================================================= 

  data_frame = prepare_df(df)
  pop_up_data_frame = create_popup_df(plot)
  style_dict = prepare_style_dict(data_frame, opacity)
  map_ = make_map(gdf, geo, style_dict)
  map_ = change_style(map_)

  return map_


def scatterPlot(df, x, y, **kwargs):
    """
    Parameters
    ----------
    df: data frame 
        data frame containing at least 
        2 numeric variables to plot
    x: type string
        name of x variable to plot
    y: type string
        name of y variable to plot
        
    """
    import numpy as np
    import seaborn as sns
    import pandas as pd
    import matplotlib as mpl
    import matplotlib.pyplot as plt
    import matplotlib.cm as cm
    from math import pi

    pm10 = df.pm10
    o3 = df.o3
    ws = df.ws
    wd = df.wd
    nox = df.nox
    no2 = df.no2

    #########################################
    # converts wind data to randians
    df = pd.DataFrame({"speed": ws, "direction": wd})
    df["speed_x"] = df["speed"] * np.sin(df["direction"] * pi / 180.0)
    df["speed_y"] = df["speed"] * np.cos(df["direction"] * pi / 180.0)
    fig, ax = plt.subplots(figsize=(8, 8), dpi=80)
    x0, x1 = ax.get_xlim()
    y0, y1 = ax.get_ylim()
    ax.set_aspect("equal")
    _ = df.plot(kind="scatter", x="speed_x", y="speed_y", alpha=0.35, ax=ax)

    ####################################
    # simple seaborn plot that shows how given variables relate with one another
    if x == "nox":
        x = nox
    elif x == "no2":
        x = no2
    elif x == "o3":
        x = o3
    elif x == "pm10":
        x = pm10
    if y == "nox":
        y = nox
    elif y == "no2":
        y = no2
    elif y == "o3":
        y = o3
    elif y == "pm10":
        y = pm10

    sns.jointplot(x=x, y=y, kind="hex")

    plt.show()


# =============================================================================
# df = pd.read_csv("mydata.csv")
# scatterPlot(df,'nox','no2')
# =============================================================================



import locale
import matplotlib as mpl
from matplotlib import docstring
import numpy as np
import pandas as pd
import random
from matplotlib.projections.polar import PolarAxes
from numpy.lib.twodim_base import histogram2d
import matplotlib.pyplot as plt

ZBASE = -1000  # The starting zorder for all drawing, negative to have the grid on
VAR_DEFAULT = "speed"
DIR_DEFAULT = "direction"
FIGSIZE_DEFAULT = (8, 8)
DPI_DEFAULT = 80
CALM_CIRCLE_COLOR = "red"
CALM_CIRCLE_ALPHA = 0.4


class WindroseAxes(PolarAxes):
    """Create a windrose axes
    """

    name = "windrose"

    def __init__(self, *args, **kwargs):
        """
        See Axes base class for args and kwargs documentation
        """

        # Uncomment to have the possibility to change the resolution directly
        # when the instance is created
        # self.RESOLUTION = kwargs.pop('resolution', 100)
        self.rmax = kwargs.pop("rmax", None)
        PolarAxes.__init__(self, *args, **kwargs)
        self.set_aspect("equal", adjustable="box", anchor="C")
        self.radii_angle = 67.5
        self.cla()

    @staticmethod
    def from_ax(ax=None, fig=None, rmax=None, *args, **kwargs):
        """
        Return a WindroseAxes object for the figure `fig`.
        """
        if ax is None:
            if fig is None:
                fig = plt.figure(
                    figsize=FIGSIZE_DEFAULT,
                    dpi=DPI_DEFAULT,
                    facecolor="w",
                    edgecolor="w",
                )
            rect = [0.1, 0.1, 0.8, 0.8]
            ax = WindroseAxes(fig, rect, facecolor="w", rmax=rmax, *args, **kwargs)
            fig.add_axes(ax)
            return ax
        else:
            return ax

    def cla(self):
        """
        Clear the current axes
        """
        PolarAxes.cla(self)

        self.theta_angles = np.arange(0, 360, 45)
        self.theta_labels = ["E", "N-E", "N", "N-W", "W", "S-W", "S", "S-E"]
        self.set_thetagrids(angles=self.theta_angles, labels=self.theta_labels)

        self._info = {"dir": list(), "bins": list(), "table": list()}

        self.patches_list = list()

        self.calm_count = None

    def _colors(self, cmap, n):
        """
        Returns a list of n colors based on the colormap cmap
        """
        return [cmap(i) for i in np.linspace(0.0, 1.0, n)]

    def set_radii_angle(self, **kwargs):
        """
        Set the radii labels angle
        """

        kwargs.pop("labels", None)
        angle = kwargs.pop("angle", None)
        if angle is None:
            angle = self.radii_angle
        self.radii_angle = angle
        N = 5
        rmax = self.get_rmax()
        radii = np.linspace(0, rmax, N + 1)
        if rmax % N == 0:
            fmt = "%d"
        else:
            fmt = "%.1f"
        radii_labels = [fmt % r for r in radii]
        # radii_labels[0] = ""  # Removing label 0
        self.set_rgrids(
            radii=radii[1:], labels=radii_labels[1:], angle=self.radii_angle, **kwargs
        )

    def _update(self):
        if not self.rmax:
            self.rmax = np.max(np.sum(self._info["table"], axis=0))
        calm_count = self.calm_count or 0
        self.set_rmax(rmax=self.rmax + calm_count)
        self.set_radii_angle(angle=self.radii_angle)

    def legend(self, pollutantName, loc="upper right", decimal_places=1, **kwargs):
        """
        Sets the legend location and her properties.
        
        Parameters
        ----------
        loc : int, string or pair of floats, default: 'upper right'
            see :obj:`matplotlib.pyplot.legend`.
        decimal_places : int, default 1
            The decimal places of the formated legend
        
        Other Parameters
        ----------------
        isaxes : boolean, default True
            whether this is an axes legend
        prop : FontProperties(size='smaller')
            the font property
        borderpad : float
            the fractional whitespace inside the legend border
        shadow : boolean
            if True, draw a shadow behind legend
        labelspacing : float, 0.005
            the vertical space between the legend entries
        handlelenght : float, 0.05
            the length of the legend lines
        handletextsep : float, 0.02
            the space between the legend line and legend text
        borderaxespad : float, 0.02
            the border between the axes and legend edge
        kwarg
            Every other kwarg argument supported by
            :obj:`matplotlib.pyplot.legend`

        """

        def get_handles():
            handles = list()
            handles.append(
                mpl.patches.Rectangle(
                        (0, 0), 0.2, 0.2, facecolor='none', edgecolor='none'
                    ))
            for p in self.patches_list:
                if isinstance(p, mpl.patches.Polygon) or isinstance(
                    p, mpl.patches.Rectangle
                ):
                    color = p.get_facecolor()
                elif isinstance(p, mpl.lines.Line2D):
                    color = p.get_color()
                else:
                    raise AttributeError("Can't handle patches")
                handles.append(
                    mpl.patches.Rectangle(
                        (0, 0), 0.2, 0.2, facecolor=color, edgecolor="black"
                    )
                )
            return handles

        def get_labels(pollutantName, decimal_places=1):
            _decimal_places = str(decimal_places)

            fmt = "[%." + _decimal_places + "f " + ": %0." + _decimal_places + "f"

            labels = np.copy(self._info["bins"])
            if locale.getlocale()[0] in ["fr_FR"]:
                fmt += "["
            else:
                fmt += ")"

            labels = [fmt % (labels[i], labels[i + 1]) for i in range(len(labels) - 1)]
            return labels

        kwargs.pop("labels", None)
        kwargs.pop("handles", None)

        # decimal_places = kwargs.pop('decimal_places', 1)

        handles = get_handles()
        labels = get_labels(decimal_places)
        labels = [f"Pollution levels ({pollutantName})"] + labels
        self.legend_ = mpl.legend.Legend(self, handles, labels, loc, **kwargs)
        return self.legend_

    def set_legend(self, **pyplot_arguments):
        if "borderaxespad" not in pyplot_arguments:
            pyplot_arguments["borderaxespad"] = -0.10
        legend = self.legend(**pyplot_arguments)
        plt.setp(legend.get_texts(), fontsize=8)
        return legend

    def _init_plot(self, direction, var, **kwargs):
        """
        Internal method used by all plotting commands
        Parameters
        ----------
        direction : 1D array,
            directions the wind blows from, North centred
        var : 1D array,
            values of the variable to compute. Typically the wind speeds
        Other Parameters
        ----------------
        normed : boolean, default False
        blowto : boolean, default False
        colors : str or list of str, default None
            The colors of the plot.
        cmap : color map, default `jet`
            A :obj:`matplotlib.cm` colormap for the plot.
            Warning! It overrides `colors`.
        weibull_factors :
        mean_values :
        frequency :
        kwarg
            Any argument accepted by :obj:`matplotlib.pyplot.plot`.
        """

        # if weibull factors are entered overwrite direction and var
        if "weibull_factors" in kwargs or "mean_values" in kwargs:
            if "weibull_factors" in kwargs and "mean_values" in kwargs:
                raise TypeError("cannot specify both weibull_factors and mean_values")
            statistic_type = "unset"
            if "weibull_factors" in kwargs:
                statistic_type = "weibull"
                val = kwargs.pop("weibull_factors")
            elif "mean_values" in kwargs:
                statistic_type = "mean"
                val = kwargs.pop("mean_values")
            if val:
                if "frequency" not in kwargs:
                    raise TypeError(
                        "specify 'frequency' argument for statistical input"
                    )
                windFrequencies = kwargs.pop("frequency")
                if len(windFrequencies) != len(direction) or len(direction) != len(var):
                    if len(windFrequencies) != len(direction):
                        raise TypeError("len(frequency) != len(direction)")
                    elif len(direction) != len(var):
                        raise TypeError("len(frequency) != len(direction)")
                windSpeeds = []
                windDirections = []
                for dbin in range(len(direction)):
                    for _ in range(int(windFrequencies[dbin] * 10000)):
                        if statistic_type == "weibull":
                            windSpeeds.append(
                                random.weibullvariate(var[dbin][0], var[dbin][1])
                            )
                        elif statistic_type == "mean":
                            windSpeeds.append(
                                random.weibullvariate(var[dbin] * 2 / np.sqrt(np.pi), 2)
                            )
                        windDirections.append(direction[dbin])
                var, direction = windSpeeds, windDirections

        # self.cla()
        kwargs.pop("zorder", None)

        # Init of the bins array if not set
        bins = kwargs.pop("bins", None)
        if bins is None:
            bins = np.linspace(np.min(var), np.max(var), 6)
        if isinstance(bins, int):
            bins = np.linspace(np.min(var), np.max(var), bins)
        bins = np.asarray(bins)
        nbins = len(bins)

        # Number of sectors
        nsector = kwargs.pop("nsector", None)
        if nsector is None:
            nsector = 12

        # Sets the colors table based on the colormap or the "colors" argument
        colors = kwargs.pop("colors", None)
        cmap = kwargs.pop("cmap", None)
        if colors is not None:
            if isinstance(colors, str):
                colors = [colors] * nbins
            if isinstance(colors, (tuple, list)):
                if len(colors) != nbins:
                    raise ValueError("colors and bins must have same length")
        else:
            if cmap is None:
                cmap = mpl.cm.jet
            colors = self._colors(cmap, nbins)

        # Building the angles list
        angles = np.arange(0, -2 * np.pi, -2 * np.pi / nsector) + np.pi / 2

        normed = kwargs.pop("normed", False)
        blowto = kwargs.pop("blowto", False)

        # Calm condition
        calm_limit = kwargs.pop("calm_limit", None)
        if calm_limit is not None:
            mask = var > calm_limit
            self.calm_count = len(var) - np.count_nonzero(mask)
            if normed:
                self.calm_count = self.calm_count * 100 / len(var)
            var = var[mask]
            direction = direction[mask]

        # Set the global information dictionnary
        self._info["dir"], self._info["bins"], self._info["table"] = histogram(
            direction, var, bins, nsector, normed, blowto
        )

        return bins, nbins, nsector, colors, angles, kwargs

    def _calm_circle(self):
        """
        Draw the calm centered circle
        and return the initial offset for plots methods
        """
        if self.calm_count and self.calm_count > 0:
            circle = mpl.patches.Circle(
                (0.0, 0.0),
                self.calm_count,
                transform=self.transData._b,
                color=CALM_CIRCLE_COLOR,
                alpha=CALM_CIRCLE_ALPHA,
            )
            self.add_artist(circle)
        return self.calm_count or 0

    def bar(self, direction, var, **kwargs):
        """
        Plot a windrose in bar mode. For each var bins and for each sector,
        a colored bar will be draw on the axes.
        Parameters
        ----------
        direction : 1D array
            directions the wind blows from, North centred
        var : 1D array
            values of the variable to compute. Typically the wind speeds.
        Other Parameters
        ----------------
        nsector : integer, optional
            number of sectors used to compute the windrose table. If not set,
            nsectors=12, then each sector will be 360/12=22.5°, and the
            resulting computed table will be aligned with the cardinals points.
        bins : 1D array or integer, optional
            number of bins, or a sequence of bins variable. If not set, bins=6
            between min(`var`) and max(`var`).
        blowto : bool, optional.
            if True, the windrose will be pi rotated, to show where the wind
            blow to (usefull for pollutant rose).
        colors : string or tuple, optional
            one string color ('k' or 'black'), in this case all bins will be
            plotted in this color; a tuple of matplotlib color args (string,
            float, rgb, etc), different levels will be plotted
            in different colors in the order specified.
        cmap : a cm Colormap instance from :obj:`matplotlib.cm`, optional.
            if cmap == None and colors == None, a default Colormap is used.
        edgecolor : string, optional
            The string color each edge box will be plotted.
            Default : no edgecolor
        opening : float, optional
            between 0.0 and 1.0, to control the space between each sector (1.0
            for no space)
        """

        bins, nbins, nsector, colors, angles, kwargs = self._init_plot(
            direction, var, **kwargs
        )
        kwargs.pop("facecolor", None)
        edgecolor = kwargs.pop("edgecolor", None)
        if edgecolor is not None:
            if not isinstance(edgecolor, str):
                raise ValueError("edgecolor must be a string color")
        opening = kwargs.pop("opening", None)
        if opening is None:
            opening = 0.8
        dtheta = 2 * np.pi / nsector
        opening = dtheta * opening

        offs = self._calm_circle()

        for j in range(nsector):
            offset = offs
            for i in range(nbins):
                if i > 0:
                    offset += self._info["table"][i - 1, j]
                val = self._info["table"][i, j]
                zorder = ZBASE + nbins - i
                patch = mpl.patches.Rectangle(
                    (angles[j] - opening / 2, offset),
                    opening,
                    val,
                    facecolor=colors[i],
                    edgecolor=edgecolor,
                    zorder=zorder,
                    **kwargs
                )
                self.add_patch(patch)
                if j == 0:
                    self.patches_list.append(patch)
        self._update()


def histogram(direction, var, bins, nsector, normed=False, blowto=False):
    if len(var) != len(direction):
        raise ValueError("var and direction must have same length")

    angle = 360.0 / nsector

    dir_bins = np.arange(-angle / 2, 360.0 + angle, angle, dtype=np.float)
    dir_edges = dir_bins.tolist()
    dir_edges.pop(-1)
    dir_edges[0] = dir_edges.pop(-1)
    dir_bins[0] = 0.0

    var_bins = bins.tolist()
    var_bins.append(np.inf)

    if blowto:
        direction = direction + 180.0
        direction[direction >= 360.0] = direction[direction >= 360.0] - 360

    table = histogram2d(x=var, y=direction, bins=[var_bins, dir_bins], normed=False)[0]
    # add the last value to the first to have the table of North winds
    table[:, 0] = table[:, 0] + table[:, -1]
    # and remove the last col
    table = table[:, :-1]
    if normed:
        table = table * 100 / table.sum()

    return dir_edges, var_bins, table


# print(mydata.head())


def windRose(df, pollutant):
    """ The plots show the proportion (here represented 
        as a percentage) of time that the wind is from 
        a certain angle and wind speed range.  
        
        Parameters
        ----------
        df: data frame
            data frame that contains wind speed
            and wind direction
        pollutant: type string
            Name of pollutant for plot to graph
    """
    pollutantSeries = df[pollutant]

    ax = WindroseAxes.from_ax()
    wd = df.wd
    ax.bar(
        wd,
        pollutantSeries,
        normed=True,
        opening=0.8,
        edgecolor="white",
        bins=[0, 10, 20, 30, 40, 398],
        cmap=plt.cm.jet,
    )
    mean_values = True
    ax.set_legend(pollutantName = pollutant)
    ax.set_title("Distribution of Pollution Levels w.r.t. Wind Direction")
    return ax


# =============================================================================
# df = pd.read_csv('mydata.csv')
# windRose(df,'pm25')
# =============================================================================



def timePlot(df, year, month, 
    pollutants=["ws", "nox", "o3", "pm25", "pm10"]):
    """
    Plot time series of pollutants for given month and year.
        
    Parameters
    ----------
    df: data frame
        a data frame of time series. Must have a date field
        and at least one variable to plot
    year: str
        year of which data will be cut
    month: int
        month of what plot will be graphed
    pollutants: list
        column names of pollutatnts to compare
    """
    import numpy as np
    import pandas as pd
    import matplotlib as mpl
    import matplotlib.pyplot as plt

    # Cuts the df down to the month specified
    df.index = pd.to_datetime(df.date)
    df = df.drop("date", axis=1)
    df_n = df[year]
    df_n = df_n.fillna(method="ffill")
    df_n["month"] = df_n.index.month
    df_n.index.dayofweek
    df_n_1 = df_n[df_n.month == month]
    # New lists that have the value of the pollutant in the month specified

    color_list = ["red", "blue", "green", "purple", "orange"]

    plt.figure(1)
    # series of `len(pollutants)` plots in one large plot that contains the
    # time series of the polutants

    axs = []

    for ix, pollutant in enumerate(pollutants):
        values = df_n_1[pollutant]
        color = color_list[ix % len(color_list)]

        # plotting
        plt.subplot(f"{len(pollutants)}1{ix}")
        a = values.plot.line(color=color)
        a.axes.get_xaxis().set_visible(False)
        a.yaxis.set_label_position("right")
        axs.append(a)
        plt.ylabel(pollutant)

    # making dates visible.
    axs[0].axes.get_xaxis().set_visible(True)
    return axs



import matplotlib.pyplot as plt
import numpy as np
import pandas as pd

# =============================================================================
# Needs work, data is random at the moment
# Not fully functional at the moment
# =============================================================================

df = pd.read_csv("mydata.csv")
mydata = pd.read_csv("mydata.csv")

pm10 = mydata.pm10
o3 = mydata.o3
ws = mydata.ws
wd = mydata.wd
nox = mydata.nox
no2 = mydata.no2
pm25 = mydata.pm25


theta = np.linspace(0, 2 * np.pi)
r = np.linspace(2, 15, 16)

Theta, R = np.meshgrid(theta, r)
C = np.sinc(Theta - 2) + (5 - np.sqrt(R)) + np.random.rand(len(r), len(theta))
C = np.ma.masked_less_equal(C, 2)
# print(C)
fig, ax = plt.subplots(figsize=(10, 10), subplot_kw={"projection": "polar"})

ax.pcolormesh(Theta, R, C, vmin=2, vmax=5, cmap="Spectral_r")

plt.show()
# print(Theta)
# print(r)



def TheilSen(df, pollutant):
    """Connected scatter plot.

    Plots a connected scatter plot of the average value of
    the pollutant every month of every year. Then plots a
    line of best fit through the plot showing the user
    the overall trend of the pollutant through the years.
    
    Parameters
    ----------
    df: data frame
        minimally containing date and at least one other
        pollutant 
    pollutant: type string
        A pollutant name correspoinding to 
        a variable in a data frame, ex: 'pm25'

    """
    import datetime as dt
    import matplotlib.pyplot as plt
    import matplotlib as mpl
    import numpy as np
    import pandas as pd
    from numpy import array
    import seaborn as sns
    import scipy
    from scipy import stats
    import math

    # =============================================================================
    # df = pd.read_csv("mydata.csv")
    # =============================================================================
    df.index = pd.to_datetime(df.date)
    unique_years = np.unique(df.index.year)
    # df = df[pd.notnull(df[pollutant])]

    i = 0
    year = []
    while i < len(unique_years):
        year.append(str(unique_years[i]))
        i = i + 1
    num_unique_years = len(year)

    # df = df.drop("date", axis=1)
    # print(df)
    i = 0
    x = 0
    j = 0
    var2 = []
    while i < num_unique_years:
        df_new = df[year[j]].resample("1D").mean()
        df_new = df_new.fillna(method="ffill")
        df_new["month"] = df_new.index.month
        # df_new['day']=df_new.index.dayofweek
        # df_new['hour']=df_new.index.hour
        i = i + 1
        j = j + 1
        x = 0
        while x < 12:
            a = df_new[df_new.month == x]
            mean_var2 = a[pollutant].mean()
            var2.append(mean_var2)
            x = x + 1
    i = 0
    while i < len(var2):
        if pd.notnull(var2[i]) == False:
            var2[i] = (var2[i - 1] + var2[i + 1]) / 2
        i = i + 1

    scatterX = []
    t = 0
    while t < num_unique_years:
        r = 0
        while r < 12:
            scatterX.append(t + (r / 12))
            r = r + 1
        t = t + 1

    y = var2
    x = scatterX

    def best_fit(X, Y):

        xbar = sum(X) / len(X)
        ybar = sum(Y) / len(Y)
        n = len(X)  # or len(Y)

        numer = sum([xi * yi for xi, yi in zip(X, Y)]) - n * xbar * ybar
        denum = sum([xi ** 2 for xi in X]) - n * xbar ** 2

        b = numer / denum
        a = ybar - b * xbar

        # print('best fit line:\ny = {:.2f} + {:.2f}x'.format(a, b))

        return a, b

    a, b = best_fit(x, y)

    fig = plt.figure()
    ax = fig.add_subplot(111)
    # print(len(x))
    ax.plot(x, y, "-o")
    ax.set_xlabel("Year")
    ax.set_ylabel(pollutant)
    ax.set_title("TheilSen plot")
    plt.plot(np.unique(x), np.poly1d(np.polyfit(x, y, 1))(np.unique(x)), color="red")
    plt.show()


# =============================================================================
# df = pd.read_csv("mydata.csv")
# TheilSen(df, 'o3')
# =============================================================================
